//   RW Dollnet
// ==============
// External, Inter-Networking Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/dollhouse/net/dollnet_util.hpp
// -Declarations of some baseline RW Dollnet utility functions

#pragma once

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

#include "curl_easy.h"

namespace rw::dollhouse::net {

//------------------------------------------------------------------------------

constexpr int tot_ticks{42};  // how wide the progress meter bar is (in chars)

constexpr float KiB{1'024.f};
constexpr float MiB{KiB * KiB};

//------------------------------------------------------------------------------
/**
 */
enum class Dir {
  up,
  dn,
};

//------------------------------------------------------------------------------
/**
 * @param xfer_frac
 * @return
 */
auto pct(float const xfer_frac) -> std::string;

//------------------------------------------------------------------------------
/**
 * @param xfer_frac
 * @param do_rev optional flag, reverse bar direction
 * @return
 */
auto ticks(float const xfer_frac, bool const do_rev = false) -> std::string;

//------------------------------------------------------------------------------
/**
 * @param xfer_tot
 * @param xfer_done
 * @param units
 * @return
 */
auto m_of_n(curl_off_t const xfer_tot,
            curl_off_t const xfer_done,
            float const      units) -> std::string;

//------------------------------------------------------------------------------
/**
 * @param durn
 * @return
 */
auto hms_fmt(Durn durn) -> std::string;

//------------------------------------------------------------------------------
/**
 * @return
 * @note: simple wrapper providing now() time call
 */
auto hms() -> std::string;

//------------------------------------------------------------------------------
/** The actual 'print the bar' code itself
 * @param xfer_tot
 * @param xfer_cmplt
 * @param dir
 * @param do_rev optional flag, reverse bar 'flow' direction
 */
void prt_bar(curl_off_t const xfer_tot,
             curl_off_t const xfer_cmplt,
             Dir const        dir,
             bool const       do_rev = false);

//------------------------------------------------------------------------------
/** Prints running count of bytes transferred over an unbounded (streaming)
 * transfer, out to console
 * @param xfer_done
 * @param dir
 * @note: bytes output as measured in MiB
 */
void prt_bytes(curl_off_t const xfer_done, Dir const dir);

//------------------------------------------------------------------------------
/** Prints 'progress meter bars' for a @b curl transfer request, out to console
 *
 * This function acts as a callback, called at @b curl 's discretion while
 * transfers are in progress
 *
 * @param unused
 * @param dl_tot
 * @param dl_rcvd
 * @param ul_tot
 * @param ul_sent
 * @return default '0' value, unused
 * @note: transfer's status data are supplied by the @b curl library itself,
 * we are simply providing for the required form for it here and utilizing
 * them
 */
auto prt_prog(Address,
              curl_off_t dl_tot  = 0,
              curl_off_t dl_rcvd = 0,
              curl_off_t ul_tot  = 0,
              curl_off_t ul_sent = 0) -> int;

//------------------------------------------------------------------------------
/** Prints elapsed time for a @b curl transfer request, out to console
 *
 * This function acts as a callback, called at @b curl 's discretion while
 * transfers are in progress
 *
 * @param unused
 * @param unused
 * @param unused
 * @param unused
 * @param unused
 * @return unused
 */
auto prt_time(Address, curl_off_t, curl_off_t, curl_off_t, curl_off_t) -> int;

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "This apartment has ultra-high bandwidth illegal even to large corporations?"

// rw_dollhouse_lib.v220326
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
