//   RW Jsonutils
// ================
// ______
// -The RW Foundational Libraries

// Filename: ./include/rw/json/json_util.hpp
// -Declarations of some baseline RW Jsonutils features

#pragma once

#include "incls_all.hpp"

namespace rw::json {

//------------------------------------------------------------------------------
/** Deserialize file stream into JSON object
 * @param json_ifs the @c std::fstream to deserialize
 * @param close_strm optional flag to close @a json_ifs; default==true
 * @return new, now-populated JSON object
 * @note: this function expects @a json_ifs parm to be a legitimate JSON file;
 * will throw/catch a specialization of std::exception on a parsing error.
 * doesn't re-throw, but @a .clear() 's the return @c Json.
 */
auto fstream_2_json(std::fstream& json_ifs, bool const close_strm = true)
    -> Json;

//------------------------------------------------------------------------------
/** Return parsed copy of JSON data contained in @a filepath file
 * @param filepath
 * @return
 */
auto rd_json_file(std::string const& filepath) -> Json;

//------------------------------------------------------------------------------
/** Write JSON object out to file
 * @param json
 * @param filepath
 * @return
 */
auto wrt_json_file(Json const& json, std::string const& filepath) -> bool;

//------------------------------------------------------------------------------
/** Attempt an @c .emplace_back() on JSON object @a j
 * @return true on success
 */
template <typename T>
auto try_emplace_back(Json& json, T const& typ) -> bool;

//------------------------------------------------------------------------------
/** Attempt an @c .at() on JSON @a tag (key) presumably contained in @a idx_it
 * @param idx_it
 * @param tag
 * @return read/write JSON reference on success
 */
auto try_idx_tag(Json::iterator idx_it, std::string const& tag) -> Json&;

}  // namespace rw::json

//    --=^=--

// "So is it true Sempai? Are robowaifus really better than any human woman?"

// rw_foundations_lib.v220326
// Copyright (2022)
// License MIT(Expat) https://opensource.org/licenses/MIT
