//   RW Sysutils
// ================
// ______
// -The RW Foundational Libraries

// Filename: ./include/rw/util/utility.hpp
// -Declarations of some baseline RW Sysutils features

#pragma once

#include "incls_all.hpp"

namespace rw::util {

//------------------------------------------------------------------------------
/**
 * @param s
 */
void rw_error(std::string const& s);

//------------------------------------------------------------------------------
/**
 * @param err_msg
 * @param tag
 * @param func_nm
 */
void rw_error(std::string const& err_msg,
              Hip const&         tag,
              std::string const& func_nm = "");

//------------------------------------------------------------------------------
/** All *slave object* classes' guarded functions should error-out on any
 * @a 'non-Master' attempt
 * @param[in] master
 * @param[in] stranger
 * @param[in] slave
 * @param[in] func_nm
 * @param[in] complaint optional
 * @note: wrapper that throws a @c std::runtime_error
 */
void only_master_can(
    Hip const&         master,
    Hip const&         stranger,
    Hip const&         slave,
    std::string const& func_nm,
    std::string const& complaint = "Only Master can ask me to do that");

//------------------------------------------------------------------------------
/** Appends a nanosecond-scaled breakdown of the passed in @c Time parm, @a tim
 * @param[in/out] os the stream to append to
 * @param[in] tim the @c Time to display
 * @return the now-appended @c ostream
 */
auto operator<<(std::ostream& os, Time const& tim) -> std::ostream&;

//------------------------------------------------------------------------------
/** Perform call analyses of the active @c RW object's function interactions
 * @note: for now, this simply dumps the entire @c Callgraph out to console,
 * with no further analysis provided
 */
void analyze_callgraph();

//------------------------------------------------------------------------------
/** Prints a Hip tag's elements out to the console, with an optional message
 * @param[in] hip the Hip data struct to print
 * @param[in] msg optional description (&tc) string
 */
void prt_tag(Hip const& tag, std::string const& msg = "");

//------------------------------------------------------------------------------
/** Convert all alphabetic characters of @a in_out to lowercase, in-place
 * @param[in/out] in_out the @c std::string to convert
 */
void to_lower(std::string& in_out);

//------------------------------------------------------------------------------
/** Determine if @a ss is empty
 * @param[in] ss @c std::stringstream to check
 * @return true if @a ss is empty, false otherwise
 */
auto is_empty(std::basic_stringstream<char>& ss) -> bool;

//------------------------------------------------------------------------------
/** Determine if @a file is empty
 * @param[in] file @c std::fstream to check
 * @return true if @a file is empty, false otherwise
 */
auto is_empty(std::basic_fstream<char>& file) -> bool;

//------------------------------------------------------------------------------
/** Parse @a strm with @c std::getline(), push lines onto @c vector<string>
 * @param[in] strm @c std::stringstream to parse
 * @return now-populated @c vector<string>
 */
auto strmlines_2_vec(std::basic_stringstream<char>& strm)
    -> std::vector<std::string>;

//------------------------------------------------------------------------------
/** Parse @a strm with @c std::getline(), push lines onto @c vector<string>
 * @param[in] strm @c std::fstream to parse
 * @return now-populated @c vector<string>
 */
auto strmlines_2_vec(std::basic_fstream<char>& strm)
    -> std::vector<std::string>;

//------------------------------------------------------------------------------
/**
 * @param[in/out] os
 * @param[in] tim
 * @return
 */
auto operator<<(std::ostream& os, Time const& tim) -> std::ostream&;

//------------------------------------------------------------------------------
/**
 * @param[in/out] ss
 * @param[in] v
 * @return
 * @note: intended as @a line-by-line operation, ie w/ @c std::getline() 's
 */
auto operator<<(std::stringstream& ss, std::vector<std::string> const& v)
    -> std::stringstream&;

//------------------------------------------------------------------------------
/**
 * @param[in/out] fs
 * @param[in] v
 * @return
 * @note: intended as @a line-by-line operation, ie w/ @c std::getline() 's
 */
auto operator<<(std::fstream& fs, std::vector<std::string> const& v)
    -> std::fstream&;

//------------------------------------------------------------------------------
/**
 * @param[in/out] ss
 * @param[out] v
 * @return
 * @note: intended as @a line-by-line operation, ie w/ @c std::getline() 's
 */
auto operator>>(std::stringstream& ss, std::vector<std::string>& v)
    -> std::stringstream&;

//------------------------------------------------------------------------------
/**
 * @param[in/out] fs
 * @param[out] v
 * @return
 * @note: intended as @a line-by-line operation, ie w/ @c std::getline() 's
 */
auto operator>>(std::fstream& fs, std::vector<std::string>& v) -> std::fstream&;

//------------------------------------------------------------------------------
/** Edit @a in_out string in-place, removing each single leading char @a c, if
 * present
 * @param[in/out] in_out the input @c std::string string to trim
 * @param[in] c the target @c char to trim from @a in_out
 */
void trim_l(std::string& in_out, const char c);

//------------------------------------------------------------------------------
/** Edit @a in_out string in-place, removing each single trailing char @a c, if
 * present
 * @param[in/out] in_out the input @c std::string string to trim
 * @param[in] c the target @c char to trim from @a in_out
 */
void trim_r(std::string& in_out, const char c);

//------------------------------------------------------------------------------
/** Edit @a in_out string in-place, removing each single leading and trailing
 * char @a c, if present
 * @param[in/out] in_out the input @c std::string string to trim
 * @param[in] c the target @c char to trim from @a in_out
 */
void trim_lr(std::string& in_out, const char c);

}  // namespace rw::util

//    --=^=--

// "So is it true Sempai? Are robowaifus really better than any human woman?"

// rw_foundations_lib.v220326
// Copyright (2022)
// License MIT(Expat) https://opensource.org/licenses/MIT
