//  RW Dolldrop
// ==============
// External File Transfers Library
// -Part of the RW Foundations suite of libraries

// Filename: ./src/dollhouse/net/dolldrop.cpp
// -Definitions of the baseline RW Dolldrop sub-library

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

namespace rw::dollhouse::net {

//------------------------------------------------------------------------------
Dolldrop_b::Dolldrop_b(Hip const& tag, std::string const& handle)
    : RW_slave{{Type::Bumpmaster, Type::Curator}, handle, Type::Dolldrop} {
  stamp("Dolldrop_b()");

  bind_master(tag);
}

//------------------------------------------------------------------------------
auto Dolldrop_b::chkd_pull_g(Hip const& tag, std::string const& uri)
    -> std::fstream {
  stamp(tag, "chkd_pull_g()");

  // pull the dolldrop's header
  auto hdr = pull_hdr_g(tag, uri);

  // check if the remote source (claims to be) modified since last download
  const auto& [have_new_drop, name] = chk_droptimes(uri, hdr);

  // check if the remote source and local dolldrop sizes match
  const auto size_match = chk_dropsizes(uri, hdr);

  if (have_new_drop) {  // yes, remote file's 'last-modified' header tag differs

    // if this is a catalog pull check, just a newer remote file is good enough
    // -otherwise, also require that the size of the remote differs from local
    //
    if (is_catalog_ || (! size_match)) {  // yes let's request the dolldrop pull

      // adjust the display cursor back up one line (re: prelim header request)
      cout << "\u008D\r" << flush;

      return pull_body_g(tag, uri);
    }
  }

  // otherwise, just return the already-saved (and current) local dolldrop
  return fstream{name};
}

//------------------------------------------------------------------------------
auto Dolldrop_b::pull_hdr_g(Hip const& tag, std::string const& uri)
    -> std::stringstream {
  stamp(tag, "pull_hdr_g()");

  if (tag == master()) {
    return mailstop_.try_hdr(t(), uri);

  } else
    only_master_can(master(), tag, t(), "Dolldrop_b::pull_hdr_g()");

  ///
  return stringstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
auto Dolldrop_b::pull_body_g(Hip const& tag, std::string const& uri)
    -> std::fstream {
  stamp(tag, "pull_body_g()");

  if (tag == master()) {
    return mailstop_.try_body(t(), uri);

  } else
    only_master_can(master(), tag, t(), "Dolldrop_b::pull_body_g()");

  ///
  return fstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
auto Dolldrop_b::pull_hdr_body_g(Hip const& tag, std::string const& uri)
    -> std::stringstream {
  stamp(tag, "pull_hdr_body_g()");

  // TEST:
  prt_tag(tag, "asked to pull a dolldrop w/ hdr");
  cout << "  -requested pull address :  " << uri << "\n\n";

  if (tag == master()) {
    return mailstop_.try_hdr_body(t(), uri);

  } else
    only_master_can(master(), tag, t(), "Dolldrop_b::pull_hdr_body_g()");

  ///
  return stringstream{};  // shouldn't reach here
}

//------------------------------------------------------------------------------
void Dolldrop::do_catalog(bool const flag) {
  stamp("do_catalog()");

  is_catalog_ = flag;
}

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "______"

// rw_dolldrop_lib.v220326
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
