//  RW Mailstop
// ==============
// ______
// -Part of the RW Foundations suite of libraries

// Filename: ./src/dollhouse/net/mailstop.cpp
// -Definitions of the baseline RW Mailstop sub-library

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

#include "incls_all.hpp"

#include "mailstop_util.cpp"

// DESIGN: NOTE: RW Mailstop likely to be moved out of rw::net, over to rw::sys

namespace rw::dollhouse::net {

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::try_hdr(Hip const& tag, std::string const& uri)
    -> std::stringstream {  //

  try {
    // TEMP: return dolldrop header directly for now
    return dollnet_.dl_hdr(tag, uri);

  } catch (runtime_error const& e) {
    return stringstream{};

  } catch (...) {
    cerr << "\n  unknown error caught in try_hdr()\n\n";
    return stringstream{};
  }
}

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::try_body(Hip const& tag, std::string const& uri)
    -> std::fstream {  //

  auto const dropname = chk_dropname(uri);
  set_name(uri, dropname);

  ofstream anons_file{dropname};

  try {
    dollnet_.dl_body(tag, uri, anons_file);

  } catch (runtime_error const& e) {
    anons_file.close();
    return fstream{};

  } catch (...) {
    cerr << "\n  unknown error caught in try_body()\n\n";
    anons_file.close();
    return fstream{};
  }

  anons_file.close();

  // TEMP: return dolldrop body directly for now
  return fstream{dropname};
}

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::try_hdr_body(Hip const& tag, std::string const& uri)
    -> std::stringstream {  //

  auto const   dropname = bld_dropname();
  ofstream     anons_file{dropname};
  stringstream hdr;

  try {
    hdr = dollnet_.dl_hdr_body(tag, uri, anons_file);

  } catch (runtime_error const& e) {
    anons_file.close();
    set_name(uri, dropname);
    return stringstream{};

  } catch (...) {
    cerr << "\n  unknown error caught in try_hdr_body()\n\n";
    anons_file.close();
    set_name(uri, dropname);
    return stringstream{};
  }

  anons_file.close();
  set_name(uri, dropname);

  // TEMP: return dolldrop header directly for now
  return hdr;
}

//------------------------------------------------------------------------------
auto Dolldrop_b::Mailstop::mk_dropdir() -> bool {
  fs::create_directory(dropdir);
  return exists_fs(dropdir);
}

//------------------------------------------------------------------------------
/** Write a general usage notes text file to dolldrops directory
 * @param dropdir
 */
void wrt_tailme() {  //

  // a raw string; the body of the text file
  auto const tailtext =
      R"(//  RW Dolldrop
// ==============
// External File Transfers Library
// -Part of the RW Foundations suite of libraries

Notes:

-You can use the tail command to watch DLs progressively fill your dolldrops
 E.G., (in a terminal from the project's base directory) :

    tail -f  ./dolldrops/anons_file_001

-You can even playback a stream-capp style dolldrop 'live' (while its capturing)
 E.G. :

    vlc  ./dolldrops/anons_file_001
       or
    mpv  ./dolldrops/anons_file_042.ogg

-Protip : If you want a file extension automatically appended to your dolldrops,
 put it on the second line of the file named :  ./dolldrops/tagfile
 E.G., for OGG dolldrops, a tagfile might be edited to begin with the contents :

    000
    .ogg

 which would (on repeated use of this demo) create a series of dolldrops named :

    ./dolldrops/anons_file_001.ogg
    ./dolldrops/anons_file_002.ogg
    ./dolldrops/anons_file_003.ogg

 &tc. (btw this can be changed anytime to get a different extension next round)

Cheers.

// rw_dollhouse_lib
// Copyright (C)
// License MIT (Expat) https://opensource.org/licenses/MIT
)";

  ofstream ofs{dropdir / "tail_me.txt"};
  ofs << tailtext;
  ofs.close();
}

//------------------------------------------------------------------------------
void Dolldrop_b::Mailstop::preflight() {  //

  if (! exists_fs(dropdir)) {  // check if ./dolldrops/ directory exists

    try {
      mk_dropdir();
      wrt_tailme();

    } catch (fs::filesystem_error const& e) {
      cout << "  fs::filesystem_error : '" << e.what()
           << "' caught in Mailstop::preflight()\n\n";

    } catch (std::bad_alloc const& e) {
      cout << "  std::bad_alloc : '" << e.what()
           << "' caught in Mailstop::preflight()\n\n";
    }
  }
}

}  // namespace rw::dollhouse::net

#endif  // __has_include(<curl_easy.h>)

//    --=^=--

// "______"

// rw_dollhouse_lib.v220326
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
